import {
  CommentOutlined,
  ProjectOutlined,
  SettingOutlined,
  TeamOutlined,
  UsergroupAddOutlined,
  BankOutlined,
  GoldOutlined,
  EnvironmentOutlined,
  BranchesOutlined,
  NodeIndexOutlined,
  NotificationOutlined,
  ScheduleOutlined,
  FileTextOutlined,
  HddOutlined,
} from "@ant-design/icons";
import { Outlet } from "react-router-dom";
import AuthLayout from "@/layouts/AuthLayout";
import Login from "@/pages/login";
import Page404 from "@/pages/404";
import Role from "@/pages/role/index";
import User from "@/pages/user/index";
import UserEdit from "@/pages/user/Edit";
import CheckList from "@/pages/check";
import CheckEdit from "@/pages/check/Edit";
import OrgList from "@/pages/org/index";

import Index from "@/pages/home/index";

import PositionList from "@/pages/position/list";
import PositionEdit from "@/pages/position/edit";
import LocTypeList from "@/pages/locType/list";
import LocTypeEdit from "@/pages/locType/edit";
import IssueTypeList from "@/pages/issueType/list";
import IssueTypeEdit from "@/pages/issueType/edit";
import QuestionList from "@/pages/question/list";
import IssueList from "@/pages/issue";
import IssueDetail from "@/pages/issue/detail";
import Notice from "@/pages/notice";

import CheckStandList from "@/pages/checkStand/list";
import CheckStandEdit from "@/pages/checkStand/edit";
import CheckLocAnswer from "@/pages/checkAnswer/loc";
import CheckLocAnswerDetail from "@/pages/checkAnswer/loc/detail";
import CheckSurveyAnswer from "@/pages/checkAnswer/survey";
import CheckSurveyAnswerDetail from "@/pages/checkAnswer/survey/detail";

/**
 * meta 用来扩展自定义数据数据
 * {
 *    title: 用于页面或者菜单的标题, 没有此字段, 菜单不会显示
 *    hideInMenu: 布尔值, 如果为 false, 菜单不会显示
 *    noLayout: 布尔值, 如果为 true, 将不会使用默认布局
 *    noSiderBar: 布尔值, 如果为 true, 将没有左侧菜单栏
 *    noFooter: 布尔值, 如果为 true, 将没有底部 footer
 *    target: 字符串, 如果为 _blank, 将在新窗口打开
 *    permission: 对应服务器端权限名称
 *    menuType: ANTD 的 menuItem 的 type 属性,
 * }
 */

export const authRoutes = [
  {
    path: "",
    element: <Outlet />,
    meta: {
      title: "业务管理",
      menuType: "group",
      permission: "biz",
    },
    children: [
      {
        path: "notice",
        element: <Notice />,
        meta: {
          title: "通知公告",
          icon: <NotificationOutlined />,
          permission: "notice",
        },
      },
      {
        path: "issue",
        element: <IssueList />,
        meta: {
          title: "问 题 单",
          icon: <CommentOutlined />,
          permission: "issue",
        },
      },
      {
        path: "issue/detail",
        element: <IssueDetail />,
        meta: {
          title: "问题单详情",
          hideInMenu: true,
          permission: "issue.detail",
        },
      },
      {
        path: "check",
        element: <CheckList />,
        meta: {
          title: "模拟测评",
          icon: <ProjectOutlined />,
          permission: "check",
        },
      },
      {
        path: "check/edit",
        element: <CheckEdit />,
        meta: {
          hideInMenu: true,
          title: "模拟测评维护",
          icon: <ProjectOutlined />,
          permission: "check.edit",
        },
      },
      {
        path: "checkAnswer",
        element: <Outlet />,
        meta: {
          title: "答题记录",
          icon: <FileTextOutlined />,
        },
        children: [
          {
            path: "loc",
            element: <CheckLocAnswer />,
            meta: {
              title: "实地测评",
              icon: <HddOutlined />,
            },
          },
          {
            path: "loc/detail",
            element: <CheckLocAnswerDetail />,
            meta: {
              title: "实地答题明细",
              hideInMenu: true
            },
          },
          {
            path: "survey",
            element: <CheckSurveyAnswer />,
            meta: {
              title: "问卷调查",
              icon: <HddOutlined />,
            },
          },
          {
            path: "survey/detail",
            element: <CheckSurveyAnswerDetail />,
            meta: {
              title: "问卷答题明细",
              hideInMenu: true
            },
          },
        ],
      },
    ],
  },
  {
    path: "dict",
    element: <Outlet />,
    meta: {
      title: "基础字典",
      icon: <ProjectOutlined />,
      menuType: "group",
      permission: "dict",
    },
    children: [
      {
        path: "locType/list",
        element: <LocTypeList />,
        meta: {
          title: "点位分类",
          icon: <EnvironmentOutlined />,
          permission: "locType",
        },
      },
      {
        path: "locType/edit",
        element: <LocTypeEdit />,
        meta: {
          hideInMenu: true,
          title: "点位分类编辑",
          // icon: <AppstoreOutlined />,
          permission: "locType.edit",
        },
      },

      {
        path: "issueType/list",
        element: <IssueTypeList />,
        meta: {
          title: "问题分类",
          icon: <BranchesOutlined />,
          permission: "issueType",
        },
      },
      {
        path: "issueType/edit",
        element: <IssueTypeEdit />,
        meta: {
          hideInMenu: true,
          title: "问题分类编辑",
          // icon: <AppstoreOutlined />,
          permission: "issueType.edit",
        },
      },
      {
        path: "question/list",
        element: <QuestionList />,
        meta: {
          title: "题库管理",
          icon: <NodeIndexOutlined />,
          permission: "question",
        },
      },
      {
        path: "checkStand/list",
        element: <CheckStandList />,
        meta: {
          title: "测评标准",
          icon: <ScheduleOutlined />,
          permission: "checkStand",
        },
      },
      {
        path: "checkStand/edit",
        element: <CheckStandEdit />,
        meta: {
          title: "测评标准",
          hideInMenu: true,
          permission: "checkStand",
        },
      },
    ],
  },
  {
    path: "system",
    element: <Outlet />,
    meta: {
      title: "系统管理",
      icon: <SettingOutlined />,
      menuType: "group",
      permission: "system",
    },
    children: [
      {
        path: "user",
        element: <User />,
        meta: {
          title: "人员管理",
          icon: <TeamOutlined />,
          permission: "user",
        },
      },
      {
        path: "user/edit",
        element: <UserEdit />,
        meta: {
          title: "人员维护",
          hideInMenu: true,
          // icon: <AppstoreOutlined />,
          permission: "user.edit",
        },
      },
      {
        path: "role",
        element: <Role />,
        meta: {
          title: "角色管理",
          icon: <UsergroupAddOutlined />,
          permission: "role",
        },
      },
      {
        path: "org",
        element: <OrgList />,
        meta: {
          title: "机构管理",
          icon: <BankOutlined />,
          permission: "org",
        },
      },
      {
        path: "position/list",
        element: <PositionList />,
        meta: {
          title: "岗位管理",
          icon: <GoldOutlined />,
          permission: "position",
        },
      },
      {
        path: "position/edit",
        element: <PositionEdit />,
        meta: {
          hideInMenu: true,
          title: "岗位管理编辑",
          // icon: <AppstoreOutlined />,
          permission: "position.edit",
        },
      },
    ],
  },
];

export const defaultRoutes = [
  {
    path: "/",
    element: <AuthLayout />,
    children: [
      {
        index: true,
        element: <Index />,
      },
      {
        path: "*",
        element: <Page404 />,
      },
    ],
  },
  {
    path: "/login",
    element: <Login />,
  },
  {
    path: "*",
    element: <Page404 />,
  },
];

export function mergeAuthRoutes(r1, r2) {
  const r = r1.slice();
  const children = r1[0].children.slice();
  r[0].children = children.concat(r2);
  return r;
}

// 全部路由
export const routes = mergeAuthRoutes(defaultRoutes, authRoutes);
function getPath(parent = "/", current = "") {
  if (current.indexOf("/") === 0 || current.indexOf("http") === 0)
    return current;
  return `${parent}/${current}`.replace(/\/\//g, "/");
}

// 路由数组, 一维数组
export const routeArr = (() => {
  const flatten = (routes, parentPath = "/") => {
    return routes.reduce((acc, route) => {
      const path = getPath(parentPath, route.path);
      const children = route.children ? flatten(route.children, path) : [];

      return acc.concat([{ ...route, path }].concat(children));
    }, []);
  };

  return flatten(routes);
})();
